package com.greatchn.collection.framework.interceptor;

import cn.hutool.json.JSONObject;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.greatchn.collection.constant.AuthConstant;
import com.greatchn.collection.framework.util.AuthUtil;
import com.greatchn.collection.framework.annotation.AuthIgnore;
import com.greatchn.collection.framework.util.DateUtil;
import com.greatchn.collection.framework.util.HttpUtil;
import com.greatchn.collection.framework.vo.ReturnMsgVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @Title AuthInterceptor
 * @Description: 权限拦截器
 * @author: wanghaohui
 * @date: 2022/6/21 8:59
 */
@Slf4j
@Component
public class AuthInterceptor implements HandlerInterceptor {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        if (request.getMethod().equalsIgnoreCase("options")) {
            return Boolean.TRUE;
        }
        // 获取request头信息中的访问标识符以及token信息

        String requestUrl = request.getRequestURL().toString();
         String token = null;
         if (requestUrl.contains("/operator")) {
            // 运维端
            token = request.getHeader(AuthConstant.HEADERS_OPERATOR_TOKEN_KEY);
            token = StringUtils.isNotBlank(token) ? token : request.getHeader(AuthConstant.HEADERS_OPERATOR_TOKEN_KEY.toLowerCase());
            token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_OPERATOR_TOKEN_KEY);
            token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_OPERATOR_TOKEN_KEY.toLowerCase());
        } else if (requestUrl.contains("/customer")) {
            // 企业端
            token = request.getHeader(AuthConstant.HEADERS_CUSTOMER_TOKEN_KEY);
            token = StringUtils.isNotBlank(token) ? token : request.getHeader(AuthConstant.HEADERS_CUSTOMER_TOKEN_KEY.toLowerCase());
            token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_CUSTOMER_TOKEN_KEY);
            token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_CUSTOMER_TOKEN_KEY.toLowerCase());
        } else if (requestUrl.contains("/bill/save")) {
            // 对外接口
             token = request.getHeader(AuthConstant.HEADERS_OUTSIDE_TOKEN_KEY);
             token = StringUtils.isNotBlank(token) ? token : request.getHeader(AuthConstant.HEADERS_OUTSIDE_TOKEN_KEY.toLowerCase());
             token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_OUTSIDE_TOKEN_KEY);
             token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_OUTSIDE_TOKEN_KEY.toLowerCase());
         } else{
             // 员工端
             token = request.getHeader(AuthConstant.HEADERS_EMPLOYEE_TOKEN_KEY);
             token = StringUtils.isNotBlank(token) ? token : request.getHeader(AuthConstant.HEADERS_EMPLOYEE_TOKEN_KEY.toLowerCase());
             token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_EMPLOYEE_TOKEN_KEY);
             token = StringUtils.isNotBlank(token) ? token : request.getParameter(AuthConstant.HEADERS_EMPLOYEE_TOKEN_KEY.toLowerCase());
         }
        // 有AuthIgnore注解的不验证
        if (handler instanceof HandlerMethod) {

            AuthIgnore authIgnore = ((HandlerMethod) handler).getMethodAnnotation(AuthIgnore.class);

            if (null != authIgnore) {

                return Boolean.TRUE;
            }
        }

        return this.verify(token, response, handler, requestUrl);
    }

    /**
     * 校验权限
     */
    public boolean verify(String token, HttpServletResponse response, Object handler, String requestUrl) throws IOException {
        // token不能为空
        if (StringUtils.isBlank(token)) {
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
        try {
            // 解析token
            JWT jwt = JWTUtil.parseToken(token);
            jwt.setKey(AuthConstant.JWT_ENCRYPT_KEY);
            JSONObject payload = jwt.getPayloads();
            if (null == payload || MapUtils.isEmpty(payload)) {
                HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
                return false;
            }
            // 过期时间
            Long expire = payload.getLong(AuthConstant.JWT_EXPIRE);
            if (null == expire || expire < DateUtil.getCurrentBeijing().getTime()) {
                HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
                return false;
            }

            if (requestUrl.contains("/operator")) {
                // 运维端登录验权限
                return this.operatorVerify(response, payload);
            } else if (requestUrl.contains("/customer")) {
                // 企业端登录验权限
                return this.customerVerify(response, payload);
            } else if (requestUrl.contains("/bill/save")) {
                // 对外接口登录权限
                return this.outSideVerify(response, payload);
            } else {
                // 员工端登录验权限
                return this.employeeVerify(response, payload);
            }
        } catch (Exception e) {
            log.error("校验权限失败, token={}, ", token, e);
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
    }

    /**
     * 运维端校验权限
     */
    public boolean operatorVerify(HttpServletResponse response, JSONObject payload) throws IOException {
        // 运维端员工id
        Long operatorId = payload.getLong(AuthConstant.JWT_OPERATOR_ID_KEY);
        if (null == operatorId || operatorId < 0) {
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
        // 将运维端员工id存入当前线程变量
        AuthUtil.setEmployeeId(operatorId);
        return true;
    }

    /**
     * 企业端校验权限
     */
    public boolean customerVerify(HttpServletResponse response, JSONObject payload) throws IOException {
        // 企业id
        Long customerId = payload.getLong(AuthConstant.JWT_CUSTOMER_ID_KEY);
        if (null == customerId || customerId < 0) {
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
        // 将信息存入当前线程变量
        AuthUtil.setCustomerId(customerId);
        return true;
    }

    /**
     * 员工端校验权限
     */
    public boolean employeeVerify(HttpServletResponse response, JSONObject payload) throws IOException {
        // 企业id
        Long customerId = payload.getLong(AuthConstant.JWT_CUSTOMER_ID_KEY);
        if (null == customerId || customerId < 0) {
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
        // 员工id
        Long employeeId = payload.getLong(AuthConstant.JWT_EMPLOYEE_ID_KEY);
        if (null == employeeId || employeeId < 0) {
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
        // 账套id
        String accountId = payload.getStr(AuthConstant.JWT_ACCOUNT_ID_KEY);
        if (StringUtils.isBlank(accountId)) {
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
        // 将信息存入当前线程变量
        AuthUtil.setCustomerId(customerId);
        AuthUtil.setEmployeeId(employeeId);
        AuthUtil.setAccountId(accountId);
        return true;
    }

    /**
     * 对外接口登录权限
     */
    public boolean outSideVerify(HttpServletResponse response, JSONObject payload) throws IOException {
        // 企业id
        Long customerId = payload.getLong(AuthConstant.JWT_CUSTOMER_ID_KEY);
        if (null == customerId || customerId < 0) {
            HttpUtil.outputResponseMsg(response, ReturnMsgVO.sessionTimeout());
            return false;
        }
        // 将信息存入当前线程变量
        AuthUtil.setCustomerId(customerId);
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        HandlerInterceptor.super.postHandle(request, response, handler, modelAndView);
    }
}
