package com.greatchn.collection.framework.util;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.greatchn.collection.framework.vo.ReturnMsgVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.util.StreamUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * http工具类
 * <p>
 * Created by duhang on 2019/8/16 17:32
 */
@Slf4j
public final class HttpUtil {

    /**
     * 连接时间
     */
    private final static int CONNECT_TIME = 5 * 60 * 1000;
    /**
     * 超时时间
     */
    private final static int OUT_TIME = 15 * 60 * 1000;

    private HttpUtil() {
    }

    /**
     * 取得request
     *
     * @return
     */
    public static HttpServletRequest getHttpServletRequest() {
        return ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
    }

    /**
     * 取得response
     *
     * @return
     */
    public static HttpServletResponse getHttpServletResponse() {
        HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Methods", "*");
        response.setHeader("Access-Control-Max-Age", "0");
        response.setHeader("Access-Control-Allow-Headers", "*");
        response.setHeader("Access-Control-Allow-Credentials", "true");
        response.setHeader("XDomainRequestAllowed", "1");
        return response;
    }

    /**
     * 跨域响应
     *
     * @param response
     * @param respObj
     * @throws IOException
     */
    public static void outputResponseMsg(HttpServletResponse response, ReturnMsgVO respObj) throws IOException {

        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Methods", "*");
        response.setHeader("Access-Control-Max-Age", "0");
        response.setHeader("Access-Control-Allow-Headers", "*");
        response.setHeader("Access-Control-Allow-Credentials", "true");
        response.setHeader("XDomainRequestAllowed", "1");

        try (PrintWriter out = response.getWriter()) {

            ObjectMapper objectMapper = SpringContextHolder.getBean(ObjectMapper.class);

            out.print(objectMapper.writeValueAsString(respObj));

            out.flush();

        } catch (IOException e) {

            throw e;
        }
    }

    /**
     * 获取请求头信息
     *
     * @param request
     * @return
     */
    public static String getReqHeader(HttpServletRequest request) {

        List<String> headerList = new ArrayList<>();

        for (Enumeration<String> enu = request.getHeaderNames(); enu.hasMoreElements(); ) {

            String name = enu.nextElement();

            headerList.add(name + " : " + request.getHeader(name));
        }

        String headers = String.join("; ", headerList);

        return headers;
    }

    /**
     * 获取rest参数
     *
     * @param request
     * @return
     */
    public static String getRestMsg(HttpServletRequest request) {

        Map<String, Object> requestParam = new HashMap<>();
        Enumeration<String> names = request.getParameterNames();
        while (names.hasMoreElements()) {
            String paramName = names.nextElement();
            String paramValue = request.getParameter(paramName);
            requestParam.put(paramName, paramValue);
        }
        if (!requestParam.isEmpty()) {

            ObjectMapper objectMapper = SpringContextHolder.getBean(ObjectMapper.class);

            try {

                return objectMapper.writeValueAsString(requestParam);

            } catch (JsonProcessingException e) {

                log.error("json转换异常, ", e);
            }
        }
        return null;
    }

    /**
     * 获取reqeustbody参数
     *
     * @param request
     * @return
     */
    public static String getBodyMsg(HttpServletRequest request) throws IOException {

        return StreamUtils.copyToString(request.getInputStream(), StandardCharsets.UTF_8);
    }

    /**
     * 获取客户端ip
     *
     * @param request
     * @return
     */
    public static String getIpAddress(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
            if ("127.0.0.1".equals(ip) || "localhost".equals(ip)) {
                //根据网卡取本机配置的IP
                InetAddress inet = null;
                try {
                    inet = InetAddress.getLocalHost();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                ip = inet.getHostAddress();
            }
        }
        // 多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        if (ip != null && ip.length() > 15) {
            if (ip.indexOf(",") > 0) {
                ip = ip.substring(0, ip.indexOf(","));
            }
        }
        return ip;
    }

    /**
     * 预览pdf
     *
     * @param request
     * @param response
     * @param source
     */
    public static void viewPDF(HttpServletRequest request, HttpServletResponse response, File source) {
        String browser = request.getHeader(HttpHeaders.USER_AGENT);
        Pattern p = Pattern.compile(".*(MSIE|Edge|Trident).*?");
        Matcher m = p.matcher(browser);
        BufferedOutputStream out = null;
        try {
            String fileName = source.getName();
            byte[] bytes = FileUtils.readFileToByteArray(source);
            response.reset();
            String userAgent = request.getHeader("user-agent").toLowerCase();
            if (m.matches() || userAgent.contains("like gecko")) {
                response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "inline;fileName=" + URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replace("+", ""));
            } else {
                response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "inline;fileName=" + new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1).replace(" ", ""));
            }
            response.setHeader("Access-Control-Allow-Origin", "*");
            response.setHeader("Access-Control-Allow-Methods", "*");
            response.setHeader("Access-Control-Max-Age", "0");
            response.setHeader("Access-Control-Allow-Headers", "*");
            response.setHeader("Access-Control-Allow-Credentials", "true");
            response.setHeader("XDomainRequestAllowed", "1");
            response.setContentType("application/pdf");
            out = new BufferedOutputStream(response.getOutputStream());
            out.write(bytes);
            out.flush();
        } catch (Exception e) {
            log.error(">>>>>>>>>>>>>>>>>>>>>>> 预览 PDF文件 出错, ", e);
            try {
                HttpUtil.outputResponseMsg(response, ReturnMsgVO.error().setMsg(e.getMessage()));
            } catch (IOException ioException) {
                log.error(" error >>>>>>>>>>", ioException);
            }
        } finally {
            if (null != out) {
                try {
                    out.close();
                } catch (IOException ignored) {
                }
            }
        }
    }

    /**
     * 下载文件
     *
     * @param request
     * @param response
     * @param source
     */
    public static void download(HttpServletRequest request, HttpServletResponse response, File source) {
        String browser = request.getHeader(HttpHeaders.USER_AGENT);
        Pattern p = Pattern.compile(".*(MSIE|Edge|Trident).*?");
        Matcher m = p.matcher(browser);
        BufferedOutputStream out = null;
        try {
            String fileName = source.getName();
            byte[] bytes = FileUtils.readFileToByteArray(source);
            response.reset();
            String userAgent = request.getHeader("user-agent").toLowerCase();
            if (m.matches() || userAgent.contains("like gecko")) {
                response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;fileName=" + URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replace("+", ""));
            } else {
                response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;fileName=" + new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1).replace(" ", ""));
            }
            // 设置文件名
            response.setHeader("Access-Control-Allow-Origin", "*");
            response.setHeader("Access-Control-Allow-Methods", "*");
            response.setHeader("Access-Control-Max-Age", "0");
            response.setHeader("Access-Control-Allow-Headers", "*");
            response.setHeader("Access-Control-Allow-Credentials", "true");
            response.setHeader("XDomainRequestAllowed", "1");
            response.setContentType("application/x-download");
            out = new BufferedOutputStream(response.getOutputStream());
            out.write(bytes);
            out.flush();
        } catch (Exception e) {
            log.error(">>>>>>>>>>>>>>>>>>>>>>> 下载 文件 出错, ", e);
            try {
                HttpUtil.outputResponseMsg(response, ReturnMsgVO.error().setMsg(e.getMessage()));
            } catch (IOException ioException) {
                log.error(" error >>>>>>>>>>", ioException);
            }
        } finally {
            if (null != out) {
                try {
                    out.close();
                } catch (IOException ignored) {
                }
            }
        }
    }


    /**
     * 下载文件
     *
     * @param request
     * @param response
     * @param bytes
     * @param fileName
     */
    public static void download(HttpServletRequest request, HttpServletResponse response, byte[] bytes, String fileName) {
        String browser = request.getHeader(HttpHeaders.USER_AGENT);
        Pattern p = Pattern.compile(".*(MSIE|Edge|Trident).*?");
        Matcher m = p.matcher(browser);
        BufferedOutputStream out = null;
        try {
            response.reset();
            String userAgent = request.getHeader("user-agent").toLowerCase();
            if (m.matches() || userAgent.contains("like gecko")) {
                response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;fileName=" + URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replace("+", ""));
            } else {
                response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;fileName=" + new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1).replace(" ", ""));
            }
            // 设置文件名
            response.setHeader("Access-Control-Allow-Origin", "*");
            response.setHeader("Access-Control-Allow-Methods", "*");
            response.setHeader("Access-Control-Max-Age", "0");
            response.setHeader("Access-Control-Allow-Headers", "*");
            response.setHeader("Access-Control-Allow-Credentials", "true");
            response.setHeader("XDomainRequestAllowed", "1");
            response.setContentType("application/x-download");
            out = new BufferedOutputStream(response.getOutputStream());
            out.write(bytes);
            out.flush();
        } catch (Exception e) {
            log.error(">>>>>>>>>>>>>>>>>>>>>>> 下载 文件 出错, ", e);
            try {
                HttpUtil.outputResponseMsg(response, ReturnMsgVO.error().setMsg(e.getMessage()));
            } catch (IOException ioException) {
                log.error(" error >>>>>>>>>>", ioException);
            }
        } finally {
            if (null != out) {
                try {
                    out.close();
                } catch (IOException ignored) {
                }
            }
        }
    }

    /**
     * 请求url携带json数据
     *
     * @param urlStr
     * @param contents
     * @return
     * @throws Exception
     * @author Fli 2017-4-24
     */
    public static String openHttpByPost(String urlStr, String contents) throws Exception {
        URL url = new URL(urlStr);
        HttpURLConnection con = (HttpURLConnection) url.openConnection();
        con.setDoOutput(true);
        con.setReadTimeout(OUT_TIME);
        con.setConnectTimeout(CONNECT_TIME);
        con.setRequestMethod("POST");
        // con.setRequestProperty("Pragma:", "no-cache");
        con.setRequestProperty("Cache-Control", "no-cache");
        con.setRequestProperty("Content-Type", "application/json");
        con.setDoInput(true);

        DataOutputStream out = new DataOutputStream(con.getOutputStream());
        out.write(contents.getBytes());
        out.flush();
        int state = con.getResponseCode();
        if (state != 200) {
            return null;
        }
        con.connect();
        String line = IOUtils.toString(con.getInputStream(), "utf-8");
        return line;
    }
}
